from focs._species import *
from macros.misc import MINIMUM_DISTANCE_EMPIRE_CHECK
from species.species_macros.empire_opinions import COMMON_OPINION_EFFECTS
from species.species_macros.env import RADIATED_STANDARD_EP
from species.species_macros.focus import (
    HAS_ADVANCED_FOCI,
    HAS_INDUSTRY_FOCUS,
)
from species.species_macros.happiness import AVERAGE_HAPPINESS
from species.species_macros.industry import GREAT_INDUSTRY
from species.species_macros.influence import NO_INFLUENCE
from species.species_macros.native_fortification import DEFAULT_NATIVE_DEFENSE
from species.species_macros.population import GOOD_POPULATION
from species.species_macros.research import NO_RESEARCH
from species.species_macros.shields import STANDARD_SHIP_SHIELDS
from species.species_macros.stockpile import AVERAGE_STOCKPILE
from species.species_macros.supply import AVERAGE_SUPPLY
from species.species_macros.troops import GREAT_DEFENSE_TROOPS

Species(
    name="SP_BEIGEGOO",
    description="SP_BEIGEGOO_DESC",
    gameplay_description="SP_BEIGEGOO_GAMEPLAY_DESC",
    native=True,
    tags=[
        "ROBOTIC",
        "GOOD_POPULATION",
        "GREAT_INDUSTRY",
        "NO_INFLUENCE",
        "NO_RESEARCH",
        "AVERAGE_SUPPLY",
        "PEDIA_ROBOTIC_SPECIES_CLASS",
    ],
    foci=[
        HAS_INDUSTRY_FOCUS,
        *HAS_ADVANCED_FOCI,
    ],
    defaultfocus="FOCUS_INDUSTRY",
    likes=[
        "FOCUS_INDUSTRY",
        "MIMETIC_ALLOY_SPECIAL",
        "PLC_ALGORITHMIC_RESEARCH",
        "PLC_CENTRALIZATION",
        "PLC_CONTINUOUS_SCANNING",
        "PLC_DESIGN_SIMPLICITY",
        "PLC_ENGINEERING",
        "PLC_ISOLATION",
        "PLC_RACIAL_PURITY",
        "RESONANT_MOON_SPECIAL",
        "SPARK_FOSSILS_SPECIAL",
        "TEMPORAL_ANOMALY_SPECIAL",
        "TIDAL_LOCK_SPECIAL",
        "WORLDTREE_SPECIAL",
    ],
    dislikes=[
        "BLD_COLLECTIVE_NET",
        "BLD_SCRYING_SPHERE",
        "BLD_SPACE_ELEVATOR",
        "BLD_STOCKPILING_CENTER",
        "CRYSTALS_SPECIAL",
        "FRUIT_SPECIAL",
        "PLC_ALLIED_REPAIR",
        "PLC_ENVIRONMENTALISM",
        "PLC_INTERSTELLAR_INFRA",
        "PLC_MODERATION",
        "PLC_NATIVE_APPROPRIATION",
        "PROBIOTIC_SPECIAL",
        "SPICE_SPECIAL",
    ],
    effectsgroups=[
        *GREAT_INDUSTRY,
        NO_RESEARCH,
        *NO_INFLUENCE,
        *AVERAGE_STOCKPILE,
        *GOOD_POPULATION,
        *AVERAGE_HAPPINESS,
        COMMON_OPINION_EFFECTS("SP_BEIGEGOO"),
        *AVERAGE_SUPPLY,
        *GREAT_DEFENSE_TROOPS,
        # not for description
        *DEFAULT_NATIVE_DEFENSE,
        *STANDARD_SHIP_SHIELDS,
        EffectsGroup(
            scope=IsSource,
            activation=Planet() & Turn(high=0) & (GalaxyMaxAIAggression >= 1) & (GalaxyMonsterFrequency >= 1),
            effects=Conditional(
                condition=MINIMUM_DISTANCE_EMPIRE_CHECK & Star(type=[Blue, White, Yellow, Orange, Red]),
                effects=[CreateShip(designname="SM_TREE")],
                else_=[AddSpecial(name="VOLCANIC_ASH_SLAVE_SPECIAL"), AddSpecial(name="VOLCANIC_ASH_MASTER_SPECIAL")],
            ),
        ),
    ],
    environments=RADIATED_STANDARD_EP,
    graphic="icons/species/beige-goo.png",
)



import os
import sys
from logging import error
from shlex import split
from traceback import print_exc

from common.option_tools import HANDLERS, get_option_dict


def init_handlers(config_str, search_dir):
    try:
        handlers = split(get_option_dict()[HANDLERS])
    except KeyError:
        handlers = []

    for handler in handlers:
        module = os.path.basename(handler)[:-3]

        # There is 3 way to specify path:
        # - absolute path to module
        # - single name, then module should be in same directory as config
        # - relative path, then use python as base path.
        if os.path.exists(handler):
            module_path = os.path.dirname(handler)
        elif not os.path.dirname(handler):
            module_path = os.path.dirname(config_str)
        else:
            module_path = os.path.join(search_dir, "..", os.path.dirname(handler))
        sys.path.insert(0, module_path)
        try:
            __import__(module)
        except Exception as e:
            for p in sys.path:
                error(p)
            error("Fail to import handler %s with error %s", handler, e)
            print_exc()
            exit(1)



from focs._tech import *
from macros.base_prod import TECH_COST_MULTIPLIER

Tech(
    name="SHP_IMPROVED_ENGINE_COUPLINGS",
    description="SHP_IMPROVED_ENGINE_COUPLINGS_DESC",
    short_description="SHIP_PART_UNLOCK_SHORT_DESC",
    category="SHIP_PARTS_CATEGORY",
    researchcost=48 * TECH_COST_MULTIPLIER,
    researchturns=3,
    tags=["PEDIA_ENGINE_PART_TECHS"],
    prerequisites=["SHP_GAL_EXPLO"],
    unlock=[Item(type=UnlockShipPart, name="FU_IMPROVED_ENGINE_COUPLINGS"), Item(type=UnlockPolicy, name="PLC_CHARGE")],
    graphic="icons/ship_parts/engine-1.png",
)



"""
Log entry samples.

##EmpireID:str:2 Name: Binding_2_pid_2_AI_1_RIdx_4_Aggressive Turn: 2
##EmpireColors:0, 255, 0, 255
##CapitalID:str:927 Name: Imperial Arbol I Species: SP_REPLICON
"""
from typing import Optional

from common.statistic_interface import LOG_PREFIX, StatKey


def _get_token_from_line(line: str) -> Optional[tuple[StatKey, str]]:
    """
    Return token from line, if not token present return None.

    Line sample:
    12:06:13.179169 {0x00002e80} [debug] python : statistic.py:30 : ##EmpireID:empire_id: 2, name: Binding_2_pid_2_AI_1_RIdx_4_Aggressive, turn: 1
    """
    index = line.find(LOG_PREFIX, 45, 80)
    if index == -1:
        return None
    line = line[index + 2 :]
    key_, val = line.split(":", 1)
    return StatKey.get_by_value_name(key_), val


def tokenize_log(path):
    with open(path) as f:
        for line in f:
            token = _get_token_from_line(line)
            if not token:
                continue
            yield token



from focs._effects import (
    EmpireHasAdoptedPolicy,
    IsSource,
    Source,
    StatisticIf,
)
from focs._tech import *
from macros.base_prod import TECH_COST_MULTIPLIER

Tech(
    name="PRO_ZERO_GEN",
    description="PRO_ZERO_GEN_DESC",
    short_description="THEORY_SHORT_DESC",
    category="PRODUCTION_CATEGORY",
    researchcost=840
    * TECH_COST_MULTIPLIER
    * (
        1
        - 0.25
        * StatisticIf(float, condition=IsSource & EmpireHasAdoptedPolicy(empire=Source.Owner, name="PLC_INDUSTRIALISM"))
    ),
    researchturns=7,
    tags=["PEDIA_PRODUCTION_CATEGORY", "THEORY"],
    prerequisites=["PRO_NDIM_ASSMB", "PRO_SINGULAR_GEN"],
    graphic="icons/tech/zero_point_energy.png",
)



from focs._species import *
from species.species_macros.empire_opinions import COMMON_OPINION_EFFECTS
from species.species_macros.env import BARREN_STANDARD_EP
from species.species_macros.focus import (
    HAS_ADVANCED_FOCI,
    HAS_INDUSTRY_FOCUS,
)
from species.species_macros.happiness import AVERAGE_HAPPINESS
from species.species_macros.industry import GOOD_INDUSTRY
from species.species_macros.influence import NO_INFLUENCE
from species.species_macros.native_fortification import DEFAULT_NATIVE_DEFENSE
from species.species_macros.population import AVERAGE_POPULATION
from species.species_macros.research import NO_RESEARCH
from species.species_macros.shields import STANDARD_SHIP_SHIELDS
from species.species_macros.stockpile import AVERAGE_STOCKPILE
from species.species_macros.supply import BAD_SUPPLY
from species.species_macros.troops import BAD_DEFENSE_TROOPS

Species(
    name="SP_TRENCHERS",
    description="SP_TRENCHERS_DESC",
    gameplay_description="SP_TRENCHERS_GAMEPLAY_DESC",
    native=True,
    tags=["ROBOTIC", "GOOD_INDUSTRY", "NO_RESEARCH", "NO_INFLUENCE", "BAD_SUPPLY", "PEDIA_ROBOTIC_SPECIES_CLASS"],
    foci=[
        HAS_INDUSTRY_FOCUS,
        *HAS_ADVANCED_FOCI,
    ],
    defaultfocus="FOCUS_INDUSTRY",
    likes=[
        "BLD_LIGHTHOUSE",
        "BLD_SCANNING_FACILITY",
        "BLD_SCRYING_SPHERE",
        "BLD_SPATIAL_DISTORT_GEN",
        "BLD_STARGATE",
        "FOCUS_PROTECTION",
        "FRACTAL_GEODES_SPECIAL",
        "KRAKEN_NEST_SPECIAL",
        "PANOPTICON_SPECIAL",
        "PHILOSOPHER_SPECIAL",
        "PLC_ALGORITHMIC_RESEARCH",
        "PLC_ARTISAN_WORKSHOPS",
        "PLC_MODERATION",
        "PLC_TERRAFORMING",
        "PLC_TRAFFIC_CONTROL",
        "SPARK_FOSSILS_SPECIAL",
        "WORLDTREE_SPECIAL",
    ],
    dislikes=[
        "BLD_AUTO_HISTORY_ANALYSER",
        "BLD_CULTURE_LIBRARY",
        "BLD_INDUSTRY_CENTER",
        "BLD_MEGALITH",
        "BLD_PLANET_BEACON",
        "BLD_REGIONAL_ADMIN",
        "BLD_STOCKPILING_CENTER",
        "COMPUTRONIUM_SPECIAL",
        "KRAKEN_IN_THE_ICE_SPECIAL",
        "PLC_CONFORMANCE",
        "PLC_CONTINUOUS_SCANNING",
        "PLC_DESIGN_SIMPLICITY",
        "PLC_ENVIRONMENTALISM",
        "PLC_NATIVE_APPROPRIATION",
        "SPICE_SPECIAL",
    ],
    effectsgroups=[
        *GOOD_INDUSTRY,
        NO_RESEARCH,
        *NO_INFLUENCE,
        *AVERAGE_STOCKPILE,
        *AVERAGE_POPULATION,
        *AVERAGE_HAPPINESS,
        COMMON_OPINION_EFFECTS("SP_TRENCHERS"),
        *BAD_SUPPLY,
        *BAD_DEFENSE_TROOPS,
        # not for description,
        *DEFAULT_NATIVE_DEFENSE,
        *STANDARD_SHIP_SHIELDS,
    ],
    environments=BARREN_STANDARD_EP,
    graphic="icons/species/robotic-02.png",
)



from focs._tech import *
from macros.base_prod import TECH_COST_MULTIPLIER

Tech(
    name="SHP_TRANSSPACE_DRIVE",
    description="SHP_TRANSSPACE_DRIVE_DESC",
    short_description="SHIP_HULL_UNLOCK_SHORT_DESC",
    category="SHIP_HULLS_CATEGORY",
    researchcost=480 * TECH_COST_MULTIPLIER,
    researchturns=8,
    tags=["PEDIA_ROBOTIC_HULL_TECHS"],
    prerequisites=["SHP_NANOROBO_MAINT"],
    unlock=[
        Item(type=UnlockShipHull, name="SH_TRANSSPATIAL"),
        Item(type=UnlockShipPart, name="FU_TRANSPATIAL_DRIVE"),
        Item(type=UnlockBuilding, name="BLD_SHIPYARD_CON_ADV_ENGINE"),
    ],
    graphic="icons/ship_hulls/trans_spatial_hull_small.png",
)



from focs._tech import *
from macros.base_prod import TECH_COST_MULTIPLIER

Tech(
    name="SHP_XENTRONIUM_PLATE",
    description="SHP_XENTRONIUM_PLATE_DESC",
    short_description="SHIP_PART_UNLOCK_SHORT_DESC",
    category="SHIP_PARTS_CATEGORY",
    researchcost=1200 * TECH_COST_MULTIPLIER,
    researchturns=6,
    tags=["PEDIA_ARMOR_PART_TECHS"],
    prerequisites=["SHP_DIAMOND_PLATE"],
    unlock=Item(type=UnlockShipPart, name="AR_XENTRONIUM_PLATE"),
    graphic="icons/tech/armor_plating.png",
)
